package com.example.bluetoothstream;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.UUID;
import java.util.function.Function;
import java.util.stream.Stream;

public class MainActivity extends AppCompatActivity {
    // Bluetooth adapter
    BluetoothAdapter bluetoothAdapter;
    // Bluetooth device
    BluetoothDevice bluetoothDevice;
    // Bluetooth socket
    BluetoothSocket bluetoothSocket;
    // InputStream
    InputStream inputStream;

    // request code for enabling Bluetooth
    private static final int REQUEST_ENABLE_BT = 42;
    // request code for fine location permission
    private static final int PERMISSION_ACCESS_FINE_LOCATION = 43;
    // for boards with bluetooth, use this UUID. For another mobile phone, use different
    private static final UUID mUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    // device MAC address
    private static final String raspberryMacAddress = "DC:A6:32:A7:46:8D";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // check for permissions
        if (checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_DENIED) {
            String[] permissions = { Manifest.permission.ACCESS_FINE_LOCATION };
            requestPermissions(permissions, PERMISSION_ACCESS_FINE_LOCATION);
        }
        // check Bluetooth adapter
        checkBluetoothAdapter();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        switch (requestCode) {
            case PERMISSION_ACCESS_FINE_LOCATION:
                Toast.makeText(this, "Granted permission Access fine location permission", Toast.LENGTH_LONG).show();
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode) {
            case REQUEST_ENABLE_BT:
                Toast.makeText(this, "Bluetooth enable activity ", Toast.LENGTH_LONG).show();
                break;
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    private void checkBluetoothAdapter() {
        // get default Bluetooth adapter
        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (bluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not supported", Toast.LENGTH_LONG).show();
        } else {
            // request Bluetooth enable
            if (!bluetoothAdapter.isEnabled()) {
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
            }
        }
    }

    public void connectToRaspberryPi(View v) {
        Toast.makeText(getApplicationContext(), "Connecting to Raspberry Pi", Toast.LENGTH_SHORT).show();

        new Thread(new Runnable() {
            @Override
            public void run() {
                // connect to Raspberry Pi
                bluetoothDevice = bluetoothAdapter.getRemoteDevice(raspberryMacAddress);
                // create bond
                bluetoothDevice.createBond();

                try {
                    // create Bluetooth socket
                    bluetoothSocket = bluetoothDevice.createRfcommSocketToServiceRecord(mUUID);
                    // start connection
                    bluetoothSocket.connect();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                try {
                    // get input stream from Bluetooth socket
                    inputStream = bluetoothSocket.getInputStream();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                // start receiving messages
                receiveData();
            }
        }).start();
    }

    public void receiveData() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                // initialize buffer
                byte[] buffer = new byte[512];
                int bytes;

//                // bind ImageView
//                final ImageView imageView = (ImageView) findViewById(R.id.image_view);
//                // create Bitmap
//                final Bitmap bitmap = Bitmap.createBitmap(8, 8, Bitmap.Config.ARGB_8888);

                while (true) {
                    try {
                        // read input stream to buffer
                        bytes = inputStream.read(buffer);
                        // convert buffer to string
                        String msg = new String(buffer, 0, bytes);

                        Log.i("[Bluetooth] Received message", msg);

//                        // split message
//                        String[] msg_split = msg.split("|");
//
//                        // convert values to float
//                        float[] values = new float[64];
//                        for (int i = 0; i < 64; i++) {
//                            values[i] = Float.parseFloat(msg_split[i]);
//                        }
//
//                        // set bitmap pixels
//                        bitmap.setPixels(values, 0, 8, 0, 0, 8, 8);
//                        // set bitmap to ImageView
//                        runOnUiThread(new Runnable() {
//                            @Override
//                            public void run() {
//                                imageView.setImageBitmap(bitmap);
//                            }
//                        });
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }).start();
    }
}